/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.flink.table.dataview

import java.util
import java.lang.{Iterable => JIterable}

import org.apache.flink.api.common.state._
import org.apache.flink.table.api.dataview.ListView

/**
  * [[ListView]] use state backend.
  *
  * @param state list state
  * @tparam T element type
  */
class StateListView[T](state: ListState[T]) extends ListView[T] {

  override def get: JIterable[T] = state.get()

  override def add(value: T): Unit = state.add(value)

  override def addAll(list: util.List[T]): Unit = {
    val iterator = list.iterator()
    while (iterator.hasNext) {
      state.add(iterator.next())
    }
  }

  override def clear(): Unit = state.clear()
}

